import re
from copy import copy


class StringUtil:

    @staticmethod
    def underscore_to_camelcase(word, sign="_"):
        return ''.join(x.capitalize() or sign for x in word.split(sign))

    @staticmethod
    def find_replace_into_text(content: str, key_value: dict):
        for key in key_value:
            value = key_value[key]
            if not value:
                value = ""
            content = content.replace(key, value)
        return content

    @staticmethod
    def camelcase_to(text: str, to: str = "_"):
        text = str(text)
        text = text.strip()
        text = re.sub(r'(?<!^)(?=[A-Z])', to, text)
        return text

    @staticmethod
    def remove_special_character(text: str, to: str = ""):
        return re.sub(r'[^\w\s/\-]', to, text)

    @staticmethod
    def text_to_url_text(text: str):
        text = StringUtil.camelcase_to(copy(text), "-")
        text = StringUtil.find_and_replace_with(text, " ", "-")
        text = StringUtil.find_and_replace_with(text, "_", "-")
        text = StringUtil.replace_multiple_occurrence_to_single_with(text=text, to="-")
        text = StringUtil.remove_special_character(text)
        text = text.strip()
        text = text.strip("-")
        text = text.lower()
        return text

    @staticmethod
    def camelcase_to_lower(text: str, to: str = "_"):
        return StringUtil.camelcase_to(text, to).lower()

    @staticmethod
    def replace_space_with(text: str, to: str = "_"):
        return re.sub('\s+', to, text)

    @staticmethod
    def find_and_replace_with(text: str, find: any, replace: any):
        text = copy(text)
        return text.replace(find, replace)

    @staticmethod
    def human_readable(text: str):
        text = StringUtil.camelcase_to(copy(text), " ")
        text = StringUtil.find_and_replace_with(text, "-", " ")
        text = text.strip()
        text = text.title()
        return StringUtil.replace_space_with(text, " ")

    @staticmethod
    def replace_multiple_occurrence_to_single_with(text: str, to: str = "_"):
        return re.sub(f"{to}+", to, text)

    @staticmethod
    def system_readable(text: str):
        text = StringUtil.camelcase_to(copy(text), "_")
        text = StringUtil.find_and_replace_with(text, " ", "_")
        text = StringUtil.find_and_replace_with(text, "-", "_")
        text = StringUtil.replace_multiple_occurrence_to_single_with(text=text, to="_")
        text = text.strip()
        text = text.lower()
        text = re.sub(r'[^a-zA-Z0-9_]', '', text)
        return text

    @staticmethod
    def remove_leading_number(text: str):
        return re.sub("^\d+", '', text)

    @staticmethod
    def lower_first_char(text: str):
        if not text:
            return ""
        text = str(text)
        return text[0].lower() + text[1:]

    @staticmethod
    def py_underscore_name(name: str):
        name = StringUtil.lower_first_char(text=name)
        name = StringUtil.system_readable(name)
        name = StringUtil.remove_special_character(name)
        name = StringUtil.remove_leading_number(name)
        return name

    @staticmethod
    def py_hyphen_name(name: str):
        name = StringUtil.py_underscore_name(name=name)
        name = StringUtil.find_and_replace_with(text=name, find="_", replace="-")
        return name

    @staticmethod
    def py_class_name(name: str):
        name = StringUtil.py_underscore_name(name=name)
        name = StringUtil.underscore_to_camelcase(word=name)
        return name



from ppy_file_text import StringUtil


def test1():
    string_collection = [
        "Student",  "tEacher", "Student_Teacher", "student99", "99student",
        "PWeb   Student", "my name is"
    ]
    for name in string_collection:
        print(f"Original: {name}, Display Name : {StringUtil.human_readable(name)}, Class Name: {StringUtil.py_class_name(name)}, File Name: {StringUtil.py_underscore_name(name)}, Pacakge Name: {StringUtil.py_hyphen_name(name)}")


test1()



from ppy_file_text.file.ppyf_file_util import FileUtil
from ppy_file_text.text.ppyt_text_file_man import TextFileMan
from ppy_file_text.text.ppyt_string_util import StringUtil
from ppy_file_text.file.ppy_archive import Archive



from ppy_common import PPyCException
from ppy_file_text import FileUtil


class TextFileMan:
    @staticmethod
    def get_text_from_file(file_path, exception_message: str = "Invalid File", is_exception: bool = True, default=None):
        if not FileUtil.is_exist(file_path):
            if is_exception:
                raise PPyCException(exception_message)
            return default

        with open(file_path, 'r', encoding="utf-8") as file:
            return file.read()

    @staticmethod
    def write_text_to_file(file_path, text_content):
        FileUtil.delete(file_path)
        try:
            stream = open(file_path, 'w', encoding="utf-8")
            stream.write(text_content)
            stream.close()
            return True
        except Exception as e:
            return False

    @staticmethod
    def write_text_file_specific_index(file_path, index, text_content):
        try:
            stream = open(file_path, 'r+', encoding="utf-8")
            lines = stream.readlines()
            lines.insert(index, text_content)
            stream.seek(0)
            stream.writelines(lines)
            stream.close()
        except Exception as e:
            return False

    @staticmethod
    def find_replace_text_content(file_path, find_replace_list_of_dict: list):
        text_content = TextFileMan.get_text_from_file(file_path)
        for find_replace_dict in find_replace_list_of_dict:
            if "find" in find_replace_dict and "replace" in find_replace_dict:
                text_content = text_content.replace(find_replace_dict["find"], find_replace_dict["replace"])

        if text_content:
            TextFileMan.write_text_to_file(file_path, text_content)



import os
import shutil
from pathlib import Path


class FileUtil:
    @staticmethod
    def is_exist(path):
        return os.path.exists(path)

    @staticmethod
    def is_it_file(path):
        return os.path.isfile(path)

    @staticmethod
    def is_it_dir(path):
        return os.path.isdir(path)

    @staticmethod
    def get_file_extension(filename):
        if '.' in filename:
            return filename.rsplit('.', 1)[1].lower()

    @staticmethod
    def filename_only(name_with_extension):
        return Path(name_with_extension).stem

    @staticmethod
    def delete(path):
        if FileUtil.is_exist(path):
            if FileUtil.is_it_file(path):
                os.remove(path)
            elif FileUtil.is_it_dir(path):
                shutil.rmtree(path, ignore_errors=True)
            else:
                return False
        return True

    @staticmethod
    def create_directories(path):
        if not os.path.exists(path):
            os.makedirs(path)

    @staticmethod
    def rename(source, destination):
        os.rename(source, destination)

    @staticmethod
    def copy(source, destination, ignore=None):
        if os.path.isdir(source):
            return shutil.copytree(source, destination, ignore)
        else:
            return shutil.copy(source, destination)

    @staticmethod
    def join_path(*args):
        return os.path.join(*args)

    @staticmethod
    def getcwd():
        return os.getcwd()

    @staticmethod
    def file_size_into_byte(path):
        if FileUtil.is_exist(path):
            return os.stat(path).st_size
        return None

    @staticmethod
    def human_readable_file_size(size):
        B = float(size)
        KB = float(1024)
        MB = float(KB ** 2)
        GB = float(KB ** 3)
        TB = float(KB ** 4)

        if B < KB:
            return '{0} {1}'.format(B, 'B' if 0 == B > 1 else 'B')
        elif KB <= B < MB:
            return '{0:.2f} KB'.format(B / KB)
        elif MB <= B < GB:
            return '{0:.2f} MB'.format(B / MB)
        elif GB <= B < TB:
            return '{0:.2f} GB'.format(B / GB)
        elif TB <= B:
            return '{0:.2f} TB'.format(B / TB)




from setuptools import setup, find_packages
import os
import pathlib

CURRENT_DIR = pathlib.Path(__file__).parent
README = (CURRENT_DIR / "README.md").read_text()

env = os.environ.get('source')


def get_dependencies():
    dependency = []

    if env and env == "code":
        return dependency

    return dependency + ["ppy-common"]


setup(
    name='ppy-file-text',
    version='0.0.3',
    url='https://github.com/banglafighter/ppy-file-text',
    license='Apache 2.0',
    author='Bangla Fighter',
    author_email='banglafighter.com@gmail.com',
    description='PWeb Python File & Text is a library which help to perform text read write operation with file directory manipulation',
    long_description=README,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=get_dependencies(),
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
    ]
)



import os
import zipfile


class Archive:

    @staticmethod
    def create_zip(source, zip_file_path):
        zip_file = zipfile.ZipFile(zip_file_path, "w")
        source_abspath = os.path.abspath(source)
        for dirname, sub_dirs, files in os.walk(source):
            for filename in files:
                abs_name = os.path.abspath(os.path.join(dirname, filename))
                arcname = abs_name[len(source_abspath) + 1:]
                zip_file.write(abs_name, arcname)
        zip_file.close()
